// #autoload
// #name = Alternate master server support
// #version = 0.1
// #date = 2008-10-24
// #author = Ben Winslow
// #warrior = Raineth
// #description = 
// #status = beta
// #category = Rain's scripts

$ServerQueue::checkMs = 100;
$AltMaster::versionDate = "20081024";
$AltMaster::versionString = "0.1";

package altmaster {
	// === CLIENT-RELATED CODE ===
	function GMJ_Browser::runQuery(%this)
	{
		// run the standard query
		Parent::runQuery(%this);
		// then start ours
		if (!$PlayingOnline)
			GMJ_FilterText.setText("Alternate master & LAN Servers");
		altMasterQuery();
	}

	function GameGui::onWake(%this)
	{
		parent::onWake(%this);
		
		// Update browser title when running in LAN mode
		if (!$PlayingOnline)
			GM_Frame.setTitle("ONLINE/LAN GAME");
	}

	function OpenLaunchTabs(%gotoWarriorSetup)
	{
		parent::OpenLaunchTabs(%gotoWarriorSetup);
		altMasterUpdateCheck();


		// Update "LAN GAME" menu items when running in LAN mode
		if ($PlayingOnline)
			return;

		%count = LaunchTabView.tabCount();
		for (%x = 0; %x < %count; %x++) {
			if (LaunchTabView.gui[%x] $= "GameGui")
				LaunchTabView.setTabText(%x, "ONLINE/LAN GAME");
		}

		// Focus the warrior selection screen like the online client
		// does on start.
		LaunchGame("WARRIOR");
	}

	function altMasterQuery()
	{
		%req = RHTTPClient::create();
		%req.setCallbackError(altMasterQueryError);
		%req.setCallbackDone(altMasterQueryDone);
		%req.setCallbackLine(altMasterQueryLine);
		%req.setVar("v", "1");
		%req.setVar("o", $PlayingOnline);
		%req.send("GET", "t2.plugh.us", "/t2/serverlist.php");
	}

	function altMasterQueryError(%this)
	{
		if (%this.state == 5)
			error("Unable to query alternate master server: HTTP error " @ %this.httpcode SPC %this.response);
		else
			error("Unable to query alternate master server: connect failed");
	}

	function altMasterQueryLine(%this, %line)
	{
		ServerQueue::Add(%line);
	}
	
	function altMasterQueryDone(%this)
	{
		%this.destroy();
	}

	function altMasterUpdateCheck()
	{
		%req = RHTTPClient::create();
		%req.setCallbackError(altMasterUpdateError);
		%req.setCallbackDone(altMasterUpdateDone);
		%req.setVar("v", "1");
		%req.setVar("vd", $AltMaster::versionDate);
		%req.setVar("vs", $AltMaster::versionString);
		%req.setVar("o", $PlayingOnline);
		%req.send("GET", "t2.plugh.us", "/t2/updatecheck.php");
	}

	function altMasterUpdateError(%this)
	{
		if (%this.state == 5)
			error("Unable to query alternate master server: HTTP error " @ %this.httpcode SPC %this.response);
		else
			error("Unable to query alternate master server: connect failed");
	}

	function altMasterUpdateDone(%this)
	{
		%title = nextToken(%this.body, flags, "\n");
		%body = nextToken(%title, title, "\n");
		%today = formatTimeString("yymmdd");
		
		%flags = nextToken(%flags, newerversion, " ");
		%flags = nextToken(%flags, messageShow, " ");
		%flags = nextToken(%flags, messageUrgent, " ");

		if ((%messageShow && $pref::lastAMUPopup !$= %today) || %messageUrgent) {
			alxPlay(sGotMail, 0, 0, 0);
			MessageBoxOK(%title, %body, "");
			if (!%messageUrgent)
				$pref::lastAMUPopup = %today;
		}
		%this.destroy();
	}


	// === SERVER-RELATED CODE ===
	function loadMissionStage2()
	{
		parent::loadMissionStage2();
		altheartbeat();
	}

	function GameConnection::onConnect(%client, %name, %raceGender, %skin, %voice, %voicePitch)
	{
		parent::onConnect(%client, %name, %raceGender, %skin, %voice, %voicePitch);
		altheartbeat();
	}
	
	function GameConnection::onDrop(%client, %reason)
	{
		parent::onDrop(%client, %reason);
		schedule(1000, 0, altheartbeat);
	}

	function altheartbeat()
	{
		if ($altheartbeat::last != 0 && getSimTime() < $altheartbeat::last + 5000) 
			return;
		
		$altheartbeat::last = getSimTime();
		if ($altheartbeat::schedule) {
			cancel($altheartbeat::schedule);
			$altheartbeat::schedule = 0;
		}

		$altheartbeat::schedule = schedule(5 * 60 * 1000, 0, altheartbeat);

		echo("Sending alternate heartbeat...");
		%req = RHTTPClient::create();

		%req.setVar("v", "1");
		%req.setVar("p", $Host::Port);
		%req.setVar("n", $ServerName);
		%req.setVar("t", $Host::MissionType);
		%req.setVar("c", $HostGamePlayerCount);
		%req.setVar("l", $Host::MaxPlayers);
		%req.setVar("m", $currentMission);
		%req.setVar("o", $PlayingOnline);
		
		%req.setCallbackDone(altheartbeatDone);
		%req.setCallbackError(altheartbeatDone);
		
		%req.send("POST", "t2.plugh.us", "/t2/heartbeat.php");
	}
	
	function stopaltheartbeat()
	{
		if ($altheartbeat::schedule) {
			cancel($altheartbeat::schedule);
			$altheartbeat::schedule = 0;
		}
	}

	function altheartbeatDone(%this)
	{
		if (%this.error)
			error("HTTP failure on alternate master");
		else {
			%message = nextToken(%this.body, response, " ");
			if (%response $= "BAD") {
				error("Heartbeat failed: " @ %message);
			} else if (%response $= "NOTICE") {
				error("Message from master: " @ %message);
			}
		}
		
		// why does this happen?
		if (!isObject(%this))
			%this.destroy();
	}

	function DestroyServer()
	{
		stopaltheartbeat();
	}

	function ServerQueue::Add(%addr)
	{
		if (!isServerQueryActive())
			pushServerAddress(%addr);
		else {
			$ServerQueue::size++;
			$ServerQueue::List[$ServerQueue::size] = %addr;
		}
		if (!$ServerQueue::scheduleid)
			$ServerQueue.scheduleid = schedule($ServerQueue::checkMs, 0, ServerQueueCheck);
	}

	function ServerQueueCheck()
	{
		$ServerQueue::scheduleid = 0;

		if (isServerQueryActive() && $ServerQueue::size > 0) {
			$ServerQueue::scheduleid = schedule($ServerQueue::CheckMs, 0, ServerQueueCheck);
			return;
		} else if ($ServerQueue::size <= 0) {
			$ServerQueue::size = 0;
			return;
		}

		pushServerAddress($ServerQueue::List[$ServerQueue::Size]);
		$ServerQueue::List[$ServerQueue::Size] = "";
		$ServerQueue::size--;

		if ($ServerQueue::size > 0)
			$ServerQueue::scheduleid = schedule($ServerQueue::CheckMs, 0, ServerQueueCheck);
	}
	
};
activatePackage(altmaster);
